/* grfconfig.c
 *

NAME
    grfconfig - show and configure graphics device parameters

SYNOPSIS
    grfconfig [-d dev] [-l] [file] 

DESCRIPTION
    grfconfig is used to add/show monitor definitions in the internal
    database of a 'grf' driver.  The monitor definitions are taken
    from a file which has lines in the format:

    num clock wid hi dep hbs hss hse hbe ht vbs vss vse vbe vt

    grfconfig takes the following arguments:

    -d	    	Add/Show monitor database entries for the device
		specified. (ie. -d /dev/grf3)

    -l 	    	List the current monitor database.  The database
		for the specfied device will be listed after
		any modes are loaded from a definition file.

    -v	    	WHen listing the monitor database, list the
		real monitor entries as well as the user-friendly
		description.

    A filename which contains monitor definitions.  

    If you wish to set the paramters for the console mode, use
    a mode number of 'c' and a depth of '4'.

CAVEATS

    Currently only grf_cl (grf3) supports a monitor database, and it
    only supports a maximum of 8 entries. (mode num 1-8). 

 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include <sys/ioctl.h>

#include <amiga/dev/grfioctl.h>

extern char *optarg;
extern int optind;

/*
 * Yes, this code is really hacky.  I'll fix it sometime when I
 * feel really motivated. :-)
 */
int
main(ac,av)
int ac;
char **av;
{
	FILE *fp;
	char *conffile;
	char *grfdev, *cmd;
	struct grfvideo_mode gfvm, *gv;
	int x,y,grffd,dolist,verbose;
	char buf[350],c,ystr[10];

	gv = &gfvm;
	cmd = av[0];
	conffile = 0;
	grfdev = 0;
	dolist = 0;
	verbose = 0;


	while((c = getopt(ac, av, "d:lv")) != -1) {
		switch(c) {
		case 'd':
			grfdev = optarg;
			break;
		case 'l':
			dolist = 1;
			break;
		case 'v':
			verbose = 1;
			break;
		default:
			printf ("%s [-d dev] [-l] [-v] [file]\n", cmd);
			return(0);
		}
	}
	ac -= optind;
	av += optind;

	if (ac == 1)
		conffile = *av;

	if ((grffd = open(grfdev, O_RDWR)) < 0) {
		printf("%s: can't open grf device.\n", cmd);
		return(1);
	}

	if (!conffile) goto skip1;  /* yes, nasty..but I'm tired */

	if (fp = fopen(conffile, "r")) {
		while (fgets(buf, 300, fp)) {
			if (buf[0] == '#')
				continue;
			x = sscanf(buf, "%9s %d %hd %hd %hd %hd %hd %hd %hd %hd %hd %hd %hd %hd %hd",
				ystr,
				&gv->pixel_clock,
				&gv->disp_width,
				&gv->disp_height,
				&gv->depth,
				&gv->hblank_start,
				&gv->hsync_start,
				&gv->hsync_stop,
				&gv->hblank_stop,
				&gv->htotal,
				&gv->vblank_start,
				&gv->vsync_start,
				&gv->vsync_stop,
				&gv->vblank_stop,
				&gv->vtotal
				);
    	    	    	if (y = atoi(ystr))
			  	gv->mode_num = y;
    	    	    	else if (ystr[0] == 'c' && ystr[1] == 0) {
    	    	    	    	gv->mode_num = 255;
    	    	    	    	gv->depth = 4;
    	    	    	}
    	    	    	    
			gv->mode_descr[0] = 0;
			if (x == 15) {
				if (ioctl(grffd, GRFIOCSETMON, (char *)gv) < 0)
					printf("%s: ioctl failed\n",cmd);
			} else {
				printf("%s: bad monitor definition\n", cmd);
			} 
				
		}
		fclose(fp);
	} else {
		printf("%s: Cannot open configuration file.\n",cmd);
		close(grffd);
		return(1);
	}

skip1:
	if (dolist) {
		ioctl(grffd, GRFGETNUMVM, &y);
		y+=2;
		for (x=1;x<y;x++) {
			x = gv->mode_num = (x!=(y-1)) ? x : 255; /* text mode */
			if (ioctl(grffd, GRFGETVMODE, gv) >= 0) {
    	    	    	    	if (x == 255)
    	    	    	    	    printf("Console: ");
    	    	    	    	else
    	    	    	    	    printf("%2d: ", gv->mode_num);
    	    	
				printf("%dx%d",
				gv->disp_width,
				gv->disp_height);

    	    	    	    	if (x != 255)
				    printf("x%d", gv->depth);
    	    	    	    	else
    	    	    	    	    printf(" (%dx%d)",
    	    	    	    	    	gv->disp_width/8,
    	    	    	    	    	gv->disp_height/gv->depth);

    	    	    	    	printf("\t%d.%dkHz @ %dHz %s\n",
				gv->pixel_clock / (gv->htotal*8000),
				(gv->pixel_clock / (gv->htotal*800)) % 10,
				gv->pixel_clock / (gv->htotal*gv->vtotal*8),
				gv->vblank_start + 100 < gv->disp_height ?
				  "I" :
				  (gv->vblank_start - 100) > gv->disp_height ?
				  "SD" :
				  "NI"
				);

				if (verbose) {
				printf("%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
				gv->mode_num,
				gv->pixel_clock,
				gv->disp_width,
				gv->disp_height,
				gv->depth,
				gv->hblank_start,
				gv->hsync_start,
				gv->hsync_stop,
				gv->hblank_stop,
				gv->htotal,
				gv->vblank_start,
				gv->vsync_start,
				gv->vsync_stop,
				gv->vblank_stop,
				gv->vtotal
				);
				}
			}
		}
	}
    
	close(grffd);
	return(0);
}
